@@ -24,9 +24,9 @@
 #include "canvas.h"
 #include "canvas-priv.h"
 
-#include <glib.h>
-#include <glib-object.h>
+#include <gdk/gdkenumtypes.h>
 
+#define CDEBUG_TYPE cria_canvas_get_type
 #include <cdebug/cdebug.h>
 #include <helpers/gnome-canvas-helpers.h>
 
@@ -168,13 +168,11 @@ cria_canvas_class_init (CriaCanvasClass	* cria_canvas_class) {
 	GObjectClass	* g_object_class;
 	GtkWidgetClass  * widget_class;
 
-	/* setting up the object class */
+	/* setting up GObjectClass */
 	g_object_class = G_OBJECT_CLASS(cria_canvas_class);
 	g_object_class->set_property = cria_canvas_set_property;
 	g_object_class->get_property = cria_canvas_get_property;
 
-	cria_canvas_class->focus_changed = cc_focus_changed;
-
 	cria_canvas_signals[CANVAS_FOCUS_CHANGED] = g_signal_new("focus-changed",
 					CRIA_TYPE_CANVAS,
 					G_SIGNAL_RUN_LAST,
@@ -204,12 +202,15 @@ cria_canvas_class_init (CriaCanvasClass	* cria_canvas_class) {
 							    0.65,
 							    G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
 
-	/* setting up the widget class */
+	/* setting up the GtkWidget class */
 	widget_class = GTK_WIDGET_CLASS(cria_canvas_class);
 	widget_class->button_press_event = cc_button_press_event;
 	widget_class->focus              = cc_focus;
 	widget_class->focus_in_event     = cc_focus_in_event;
 	widget_class->focus_out_event    = cc_focus_out_event;
+
+	/* setting up CriaCanvasClass */
+	cria_canvas_class->focus_changed = cc_focus_changed;
 }
 
 /**
@@ -365,7 +366,7 @@ void
 cria_canvas_set_zoom(CriaCanvas* self, gdouble zoom) {
 	g_return_if_fail(CRIA_IS_CANVAS(self));
 
-	cdebugo(self, "setZoom()", "to %f", zoom);
+	cdebug("setZoom()", "to %f", zoom);
 
 	if(zoom != self->zoom) {
 		self->zoom = zoom;
@@ -397,6 +398,18 @@ cria_canvas_update_zoom(CriaCanvas* self) {
 	}
 }
 
+#if defined(G_HAVE_ISO_VARARGS) || defined(G_HAVE_GNUC_VARARGS)
+# undef cdebug
+# undef CDEBUG_TYPE
+# define CDEBUG_TYPE cria_item_get_type
+# ifdef G_HAVE_ISO_VARARGS
+#  define cdebug(nspace, message, ...) cdebugt(CDEBUG_TYPE(), nspace, __VA_ARGS__)
+# else
+#  define cdebug(nspace, message...) cdebugt(CDEBUG_TYPE(), nspace, message)
+# endif
+#else
+#endif
+
 static gdouble
 ci_point(GnomeCanvasItem* item, double x, double y, int cx, int cy, GnomeCanvasItem** actual_item) {
 	GList          * it;
@@ -538,7 +551,25 @@ ci_event(GnomeCanvasItem* item, GdkEvent* ev) {
 	case GDK_KEY_PRESS:
 		g_signal_emit(item, cria_item_signals[ITEM_KEY_PRESS_EVENT], 0, ev, &retval);
 		break;
+	case GDK_BUTTON_PRESS:
+		g_signal_emit(item, cria_item_signals[ITEM_BUTTON_PRESS_EVENT], 0, ev, &retval);
+		break;
+/*	case GDK_FOCUS_CHANGE:
+		g_signal_emit(item, cria_item_signals[ITEM_FOCUS], 0, ev, &retval);
+		break;
+#warning "CriaItem::event(): FIXME: fix focus handling"
+	case GDK_FOCUS_IN_EVENT:
+		g_signal_emit(item, cria_item_signals[ITEM_FOCUS_IN_EVENT], 0, ev, &retval);
+		break;
+	case GDK_FOCUS_OUT_EVENT:
+		g_signal_emit(item, cria_item_signals[ITEM_FOCUS_OUT_EVENT], 0, ev, &retval);
+		break;*/
 	default:
+		{
+			GTypeClass* c = g_type_class_ref(GDK_TYPE_EVENT_TYPE);
+			cdebug("event()", "unhandled event of type %s", g_enum_get_value(G_ENUM_CLASS(c), ev->type)->value_name);
+			g_type_class_unref(c);
+		}
 		break;
 	}
 	
@@ -728,6 +759,7 @@ cria_item_set_interactive(CriaItem* self, gboolean interactive) {
 		CRIA_ITEM_UNSET_FLAG(self, CRIA_INTERACTIVE);
 	}
 	
+	cdebug("setInteractive()", "set interactive to %s", interactive ? "true" : "false");
 	g_object_notify(G_OBJECT(self), "interactive");
 }
 
