@@ -23,6 +23,8 @@
 
 #include "gtk-helpers.h"
 
+#include <gtk/gtkmain.h>
+
 void
 gtk_main_flush(void) {
 	while(gtk_events_pending()) {
@@ -38,6 +40,18 @@ gtk_widget_hide_now(GtkWidget* self) {
 	gtk_main_flush();
 }
 
+void
+gtk_widget_set_cursor(GtkWidget* self, GdkCursorType type) {
+	GdkCursor* cursor;
+
+	g_return_if_fail(GTK_IS_WIDGET(self));
+
+	cursor = gdk_cursor_new(type);
+	gdk_window_set_cursor(self->window, cursor);
+	gdk_display_flush(gtk_widget_get_display(self));
+	gdk_cursor_unref(cursor);
+}
+
 gchar*
 gtk_window_get_geometry(GtkWindow* self) {
 	int x,y,w,h;
