@@ -25,9 +25,6 @@
 
 #include <string.h>
 
-#include "slide-editor.h"
-#include "slide-list.h"
-
 GtkWidget*
 criawips_libglade_custom_handler (GladeXML* xml, gchar* func_name, gchar* widget_name, gchar* string1, gchar* string2, gint int1, gint int2, gpointer data) {
 	GtkWidget	* return_widget = NULL;
