@@ -28,19 +28,23 @@
 #include <rendering/cria-block-renderer-priv.h>
 
 #include <inttypes.h>
+#include <glib/gi18n.h>
 #include <libgnomecanvas/libgnomecanvas.h>
 
 #include <helpers/gtk-helpers.h>
 #include <helpers/gnome-canvas-helpers.h>
+
+#define CDEBUG_TYPE cria_block_renderer_get_type
 #include <cdebug/cdebug.h>
 
 #include <utils/cria-units.h>
 #include <dom/cria-format.h>
+#include <dom/cria-block-view.h>
 #include <rendering/cria-canvas-text.h>
 #include <rendering/cria-format.h>
+#include <rendering/cria-slide-renderer.h>
 
 #include "application.h"
-#include "block-view.h"
 
 enum {
 	PROP_0,
@@ -51,15 +55,20 @@ enum {
 	N_SIGNALS
 };
 
-G_DEFINE_TYPE_WITH_CODE(CriaBlockRenderer, cria_block_renderer, CRIA_TYPE_ITEM,
+#define MSG_CLICK_HERE _("Click here to add text...")
+
+#ifdef WITH_CCC
+# define BASE CC_TYPE_NODE
+#else
+# define BASE CRIA_TYPE_ITEM
+#endif
+G_DEFINE_TYPE_WITH_CODE(CriaBlockRenderer, cria_block_renderer, BASE,
 			G_IMPLEMENT_INTERFACE(CRIA_TYPE_BLOCK_VIEW, NULL))
 
-static GObjectClass* parent_class = NULL;
 static	void	cria_block_renderer_get_property(GObject		* object,
 						 guint		  prop_id,
 						 GValue		* value,
 						 GParamSpec	* param_spec);
-static void cria_block_renderer_init	        (CriaBlockRenderer* self);
 static	void	cria_block_renderer_set_property(GObject		* object,
 						 guint		  prop_id,
 						 const GValue	* value,
@@ -74,6 +83,7 @@ static	void	cria_block_renderer_signal	       (CriaBlockRenderer	* self,
 						const	gchar	* string);
 #endif
 
+#ifndef WITH_CCC
 static gboolean
 cbr_button_press_event(CriaItem* self, GdkEventButton* ev) {
 	cria_item_grab_focus(self);
@@ -82,7 +92,9 @@ cbr_button_press_event(CriaItem* self, GdkEventButton* ev) {
 
 static void
 cbr_ensure_caret(CriaBlockRenderer* self) {
-	if(!self->caret) {
+	g_return_if_fail(CRIA_IS_BLOCK_RENDERER(self));
+	
+	if(!self->caret && CRIA_ITEM_IS_INTERACTIVE(self)) {
 		self->caret = gnome_canvas_item_new(GNOME_CANVAS_GROUP(self),
 						    gnome_canvas_line_get_type(),
 						    "fill-color", "black",
@@ -123,6 +135,11 @@ cbr_disable_caret_timeout(CriaBlockRenderer* self) {
 
 static void
 cbr_reset_caret(CriaBlockRenderer* self) {
+	if(!CRIA_ITEM_IS_INTERACTIVE(self)) {
+		cdebug("resetCaret()", "Not resetting the caret.");
+		return;
+	}
+	
 	cbr_ensure_caret(self);
 	cbr_disable_caret_timeout(self);
 	
@@ -137,6 +154,18 @@ cbr_reset_caret(CriaBlockRenderer* self) {
 	gtk_main_flush();
 }
 
+static void cbr_update_position(CriaBlockRenderer* self);
+
+static void
+cbr_notify_text(CriaBlockRenderer* self, GParamSpec* pspec, CriaCanvasText* cct) {
+	gchar const* text;
+	g_object_get(cct, "text", &text, NULL);
+	if(text && strcmp(text, MSG_CLICK_HERE)) {
+		cria_block_set_text(cria_block_renderer_get_block(self), text);
+	}
+	cbr_update_position(self);
+}
+
 static void
 cbr_ensure_text_item(CriaBlockRenderer* self) {
 	if(!self->text_item) {
@@ -145,13 +174,174 @@ cbr_ensure_text_item(CriaBlockRenderer* self) {
 							"clip", TRUE,
 							"size-set", TRUE,
 							NULL);
+		g_signal_connect_swapped(self->text_item, "notify::text",
+					 G_CALLBACK(cbr_notify_text), self);
 		g_signal_connect_swapped(self->text_item, "notify::cursor",
 					 G_CALLBACK(cbr_update_cursor_position), self);
-		g_signal_connect_swapped(self->text_item, "notify::cursor-trail",
-					 G_CALLBACK(cbr_update_cursor_position), self);
 	}
 }
 
+static gint
+cbr_element_names_equal(gconstpointer a, gconstpointer b) {
+	CriaSlideElement const* elem_a = CRIA_SLIDE_ELEMENT(a);
+	CriaSlideElement const* elem_b = CRIA_SLIDE_ELEMENT(b);
+
+	gchar const* name_a = cria_slide_element_get_name(elem_a);
+	gchar const* name_b = cria_slide_element_get_name(elem_b);
+	
+	gint retval = (name_a && name_b) ? strcmp(name_a, name_b) : -1;
+
+	cdebug("elementNamesEqual()", "'%s' & '%s' => %d", name_a, name_b, retval);
+
+	return retval;
+}
+
+/* cbr_get_block_template:
+ * @block: a #CriaBlock
+ * @slide: the slide that contains the block
+ * @master_slide: return location for the master slide
+ *
+ * Get the block that serves as a template for another one.
+ *
+ * Returns the block that serves as the template for @block.
+ */
+static CriaBlock const*
+cbr_get_block_template(CriaBlock const* block, CriaSlide const* slide, CriaSlide const** master_slide) {
+	CriaBlock const* template = NULL;
+	CriaSlide      * master    = cria_slide_get_master_slide(slide);
+	GList          * templates = master ? cria_slide_get_elements(master) : NULL;
+
+	GList* possible_template = g_list_find_custom(templates, block, cbr_element_names_equal);
+
+	if(possible_template && CRIA_IS_BLOCK(possible_template->data)) {
+		template = CRIA_BLOCK(possible_template->data);
+	}
+
+	g_list_free(templates);
+
+	*master_slide = master;
+	return template;
+}
+
+#define GET_RECURSIVE_FROM_BLOCK_FUNC(type, property, _RETURN_DEFAULT_) \
+static type \
+cbr_get_##property(CriaBlockRenderer const* self) {\
+	type  property = _RETURN_DEFAULT_;\
+	CriaBlock const* block = NULL;\
+	\
+	g_return_val_if_fail(CRIA_IS_BLOCK_RENDERER(self), property);\
+	\
+	block = cria_block_renderer_get_block(self);\
+	property = cria_block_get_##property(block);\
+	if(property == _RETURN_DEFAULT_) {\
+		/* return from one of the templates */\
+		CriaSlideRenderer const* csr = CRIA_SLIDE_RENDERER(GNOME_CANVAS_ITEM(self)->parent);\
+		CriaSlide const* slide       = cria_slide_renderer_get_slide(csr);\
+		CriaBlock const* template    = cbr_get_block_template(block, slide, &slide);\
+		\
+		cdebug("cbr_get_" G_STRINGIFY(property) "()", "getting value from template");\
+		\
+		for(; CRIA_IS_BLOCK(template); template = cbr_get_block_template(template, slide, &slide)) {\
+			property = cria_block_get_##property(template);\
+			if(property != _RETURN_DEFAULT_) {\
+				cdebug("cbr_get_" G_STRINGIFY(property) "()", "got %d from template", property);\
+				break;\
+			}\
+		}\
+	}\
+	\
+	return property;\
+}
+
+#define GET_RECURSIVE_FROM_BLOCK_FUNC_PTR(type, property, _RETURN_DEFAULT_) \
+static type \
+cbr_get_##property(CriaBlockRenderer const* self) {\
+	type  property = _RETURN_DEFAULT_;\
+	type const* property##_ptr = NULL;\
+	CriaBlock const* block = NULL;\
+	\
+	g_return_val_if_fail(CRIA_IS_BLOCK_RENDERER(self), property);\
+	\
+	block = cria_block_renderer_get_block(self);\
+	property##_ptr = cria_block_get_##property(block);\
+	if(property##_ptr) {\
+		property = *property##_ptr;\
+	} else {\
+		/* return from one of the templates */\
+		CriaSlideRenderer const* csr = CRIA_SLIDE_RENDERER(GNOME_CANVAS_ITEM(self)->parent);\
+		CriaSlide const* slide       = cria_slide_renderer_get_slide(csr);\
+		CriaBlock const* template    = cbr_get_block_template(block, slide, &slide);\
+		\
+		for(; CRIA_IS_BLOCK(template); template = cbr_get_block_template(template, slide, &slide)) {\
+			property##_ptr = cria_block_get_##property(template);\
+			if(property##_ptr) {\
+				property = *property##_ptr;\
+				break;\
+			}\
+		}\
+	}\
+	return property;\
+}
+
+/*
+ * cbr_get_alignment:
+ * @self: a #CriaBlockRenderer
+ *
+ * Get the alignment value that should be used for display. As this calculation
+ * needs some inheritance magic, it's been split out to an own function.
+ *
+ * Returns the alignment of the block or the first template that sets it;
+ * #CRIA_ALIGNMENT_UNSET if none was set.
+ */
+GET_RECURSIVE_FROM_BLOCK_FUNC(CriaAlignment, alignment, CRIA_ALIGNMENT_UNSET);
+
+/*
+ * cbr_get_valignment:
+ * @self: a #CriaBlockRenderer
+ *
+ * Get the vertical alignment value that should be used for display. As this
+ * calculation needs some inheritance magic, it's been split out to an own
+ * function.
+ *
+ * Returns the vertical alignment of the block or the first template that sets
+ * it; #CRIA_VALIGNMENT_UNSET if none was set.
+ */
+GET_RECURSIVE_FROM_BLOCK_FUNC(CriaVAlignment, valignment, CRIA_VALIGNMENT_UNSET);
+
+/*
+ * cbr_get_color:
+ * @self: a #CriaBlockRenderer
+ *
+ * Get the foreground color of the rendered block.
+ *
+ * Returns the foreground color of the rendered block (even if specified in a
+ * template).
+ */
+GET_RECURSIVE_FROM_BLOCK_FUNC_PTR(GOColor, color, RGBA_BLACK);
+
+/*
+ * cbr_get_format:
+ * @self: a #CriaBlockRenderer
+ *
+ * Get the format of the rendered block.
+ *
+ * Returns the format string of the rendered block (even if specified in a
+ * template).
+ */
+GET_RECURSIVE_FROM_BLOCK_FUNC(CriaLepton, format, 0);
+
+/*
+ * cbr_get_position:
+ * @self: a #CriaBlockRenderer
+ *
+ * Get the position of the rendered block.
+ *
+ * Returns the position of the rendered block (even if it's specified in one of
+ * its templates).
+ */
+static GoRect const default_rect = {0l, 0l, 0l, 0l};
+GET_RECURSIVE_FROM_BLOCK_FUNC_PTR(GoRect, position, default_rect);
+
 static void
 cbr_update_cursor_position(CriaBlockRenderer* self) {
 	gint               cursor, trail;
@@ -160,10 +350,15 @@ cbr_update_cursor_position(CriaBlockRenderer* self) {
 	gdouble            coords[2];
 	gdouble            zoom_times_pango_scale;
 
+	if(!CRIA_ITEM_IS_INTERACTIVE(self)) {
+		return;
+	}
+
 	cbr_ensure_caret(self);
 	cbr_ensure_text_item(self);
 	
 	g_object_get(self->text_item, "cursor", &cursor, "cursor-trail", &trail, NULL);
+	cdebug("updateCursorPosition()", "current cursor position is {%d+%d}", cursor, trail);
 	pango_layout_get_cursor_pos(GNOME_CANVAS_TEXT(self->text_item)->layout,
 				    cursor + trail,
 				    &r, NULL);
@@ -182,7 +377,7 @@ cbr_update_cursor_position(CriaBlockRenderer* self) {
 	p->coords[1] = r.y / zoom_times_pango_scale + p->coords[1];
 	p->coords[3] = p->coords[1] + (r.height / zoom_times_pango_scale);
 
-	switch(cria_block_get_alignment(self->block)) {
+	switch(cbr_get_alignment(self)) {
 	case CRIA_ALIGNMENT_RIGHT:
 		p->coords[0] -= coords[0];
 		p->coords[2] -= coords[0];
@@ -191,13 +386,18 @@ cbr_update_cursor_position(CriaBlockRenderer* self) {
 		p->coords[0] -= 0.5 * coords[0];
 		p->coords[2] -= 0.5 * coords[0];
 		break;
-	default:
-		/* unset, left and right need no modifications */
+	case CRIA_ALIGNMENT_LEFT:
+	case CRIA_ALIGNMENT_JUSTIFY:
+	case CRIA_ALIGNMENT_UNSET:
+		/* unset, left and justify need no modifications */
 #warning "FIXME: check whether we should follow the gtk text direction in the unset case"
 		break;
+	default:
+		g_warning("got a value for CriaAlignment that cannot be understood: %d", cbr_get_alignment(self));
+		break;
 	}
 
-	switch(cria_block_get_valignment(self->block)) {
+	switch(cbr_get_valignment(self)) {
 	case CRIA_ALIGNMENT_BOTTOM:
 		p->coords[1] -= coords[1];
 		p->coords[3] -= coords[1];
@@ -206,31 +406,19 @@ cbr_update_cursor_position(CriaBlockRenderer* self) {
 		p->coords[1] -= 0.5 * coords[1];
 		p->coords[3] -= 0.5 * coords[1];
 		break;
-	default:
+	case CRIA_ALIGNMENT_TOP:
+	case CRIA_VALIGNMENT_UNSET:
 		/* unset and top don't need modification */
 		break;
+	default:
+		g_warning("got a value for CriaVAlignment that cannot be understood: %d", cbr_get_valignment(self));
+		break;
 	}
 
 	g_object_set(self->caret, "points", p, NULL);
 	cbr_reset_caret(self);
 }
 
-static gboolean
-cbr_focus_in_event(CriaItem* item, GdkEventFocus* ev) {
-	CriaBlockRenderer* self = CRIA_BLOCK_RENDERER(item);
-
-	cbr_disable_caret_timeout(self);
-	
-	if(self->border) {
-		g_object_set(self->border,
-			     "outline-color-gdk", &(GTK_WIDGET(GNOME_CANVAS_ITEM(item)->canvas)->style->base[GTK_STATE_SELECTED]),
-			     NULL);
-		cbr_reset_caret(self);
-	}
-
-	return FALSE || CRIA_ITEM_CLASS(parent_class)->focus_in_event(item, ev);
-}
-
 static void
 cbr_border_unfocused(CriaBlockRenderer* self) {
 	g_return_if_fail(CRIA_IS_BLOCK_RENDERER(self));
@@ -255,11 +443,12 @@ cbr_focus_out_event(CriaItem* item, GdkEventFocus* ev) {
 
 	cbr_disable_caret_timeout(self);
 	
-	return FALSE || CRIA_ITEM_CLASS(parent_class)->focus_out_event(item, ev);
+	return CRIA_ITEM_CLASS(cria_block_renderer_parent_class)->focus_out_event(item, ev);
 }
+#endif
 
 CriaBlock*
-cria_block_renderer_get_block(CriaBlockRenderer* self) {
+cria_block_renderer_get_block(CriaBlockRenderer const* self) {
 	g_return_val_if_fail(CRIA_IS_BLOCK_RENDERER(self), NULL);
 	
 	return self->block;
@@ -281,17 +470,20 @@ cria_block_renderer_get_property(GObject* object, guint prop_id, GValue* value,
 	}
 }
 
+#ifndef WITH_CCC
 static void
 hide_block_border(CriaBlockRenderer* self) {
 	if(self->border) {
 		gnome_canvas_item_hide(self->border);
 	}
 }
+#endif
 
 void
 cria_block_renderer_hide(CriaBlockRenderer* self) {
 	g_return_if_fail(CRIA_IS_BLOCK_RENDERER(self));
 
+#ifndef WITH_CCC
 	if(self->text_item) {
 		cdebugo(self, "rendererHide()", "hiding item 0x%x", (uintptr_t)self->text_item);
 		gnome_canvas_item_hide(self->text_item);
@@ -300,11 +492,14 @@ cria_block_renderer_hide(CriaBlockRenderer* self) {
 	if(self->border) {
 		hide_block_border(self);
 	}
+#endif
 }
 
 static void
 cria_block_renderer_init(CriaBlockRenderer* self) {
+#ifndef WITH_CCC
 	CRIA_ITEM_SET_FLAG(self, CRIA_CAN_FOCUS);
+#endif
 }
 
 /**
@@ -316,6 +511,7 @@ cria_block_renderer_init(CriaBlockRenderer* self) {
  *
  * Returns the new block renderer.
  */
+#ifndef WITH_CCC
 CriaBlockRenderer*
 cria_block_renderer_new(CriaItem* parent, gboolean interactive) {
 	g_return_val_if_fail(CRIA_IS_ITEM(parent), NULL);
@@ -324,6 +520,7 @@ cria_block_renderer_new(CriaItem* parent, gboolean interactive) {
 							 "interactive", interactive,
 							 NULL));
 }
+#endif
 
 /* these are type_map[horiz][vert] */
 static GtkAnchorType type_map[][4] = {
@@ -390,26 +587,46 @@ get_y_from_valignment(CriaVAlignment align, go_unit_t top, go_unit_t bottom) {
 	return 0.0;
 }
 
+static gboolean
+cbr_displays_text(CriaBlockRenderer const* self) {
+	gchar const* text = cria_block_get_markup(cria_block_renderer_get_block(self));
+	gboolean retval = /* non-interactive items display what they're set to: */
+			  !CRIA_ITEM_IS_INTERACTIVE(self) ||
+			  /* focused items display their real text: */
+	                  GNOME_CANVAS_ITEM(self)->canvas->focused_item == GNOME_CANVAS_ITEM(self) ||
+			  /* non-empty text should be displayed */
+	                  (text && *text);
+#ifndef CDEBUG_DISABLED
+	{
+	gchar *msg = g_strescape(text, "");
+	cdebug("displaysText()", "interactive? %d; focused? %d; text '%s' => %d",
+	       CRIA_ITEM_IS_INTERACTIVE(self),
+	       GNOME_CANVAS_ITEM(self)->canvas->focused_item == GNOME_CANVAS_ITEM(self),
+	       msg,
+	       retval);
+	g_free(msg);
+	}
+#endif
+	return retval;
+}
+
 static void
 cbr_update_position(CriaBlockRenderer* self) {
 #warning "updatePosition(): FIXME: make this a callback for the 'notify::position' signal of the block"
-	GoRect block_position;
+	GoRect block_position = cbr_get_position(self);
 	gdouble x, y;
-
-	if(cria_block_get_position(cria_block_renderer_get_block(self))) {
-		block_position = *cria_block_get_position(cria_block_renderer_get_block(self));
-	} else {
-		block_position.top =
-			block_position.bottom =
-			block_position.left =
-			block_position.right = 0;
-	}
+	CriaAlignment  align  = CRIA_ALIGNMENT_CENTER;
+	CriaVAlignment valign = CRIA_ALIGNMENT_MIDDLE;
 
 	/* extend the block size as far as necessary to contain the whole text */
+#ifdef WITH_CCC
+# warning "FIXME: add ccc implementation"
+#else
 	g_object_get(self->text_item,
 		     "text-width", &x,
 		     "text-height", &y,
 		     NULL);
+#endif
 	if(x > (block_position.right - block_position.left)) {
 		/* extend by alignment */
 #warning "updatePosition(): FIXME: extend by alignment first"
@@ -433,10 +650,16 @@ cbr_update_position(CriaBlockRenderer* self) {
 		}
 	}
 
+	if(cbr_displays_text(self)) {
+		align  = cbr_get_alignment(self);
+		valign = cbr_get_valignment(self);
+	}
+
+#ifndef WITH_CCC
 	if(self->text_item) {
 		g_object_set(self->text_item,
-			     "x", get_x_from_alignment(cria_block_get_alignment(cria_block_renderer_get_block(self)), block_position.left, block_position.right),
-			     "y", get_y_from_valignment(cria_block_get_valignment(cria_block_renderer_get_block(self)), block_position.top, block_position.bottom),
+			     "x", get_x_from_alignment(align, block_position.left, block_position.right),
+			     "y", get_y_from_valignment(valign, block_position.top, block_position.bottom),
 			     "clip-height", 0.0 + block_position.bottom - block_position.top,
 			     "clip-width", 0.0 + block_position.right - block_position.left,
 			     NULL);
@@ -450,66 +673,71 @@ cbr_update_position(CriaBlockRenderer* self) {
 			     "y2", 0.0 + block_position.bottom,
 			     NULL);
 	}
-}
-
-static void
-update_font(CriaBlockRenderer* self, const PangoFontDescription* desc) {
-	gdouble  font_size;
-	gchar  * font_string;
-	
-	g_return_if_fail(CRIA_IS_BLOCK_RENDERER(self));
-	g_return_if_fail(desc != NULL);
-
-	cbr_ensure_text_item(self);
-
-	font_size   = 0.0 + pango_font_description_get_size(desc) * MASTER_COORDINATES_PER_POINT / PANGO_SCALE;
-	font_string = pango_font_description_to_string(desc);
-	
-	cdebugo(self, "updateFont()", "Font is now 0x%x", (uintptr_t)desc);
-	cdebugo(self, "updateFont()", "Font: %s", font_string);
-
-	g_object_set(self->text_item,
-		     "font-desc", desc,
-		     NULL);
-
-	cdebugo(self, "updateFont()", "showing canvas item at size %f", font_size);
-#warning "FIXME: make it unnecessary to set the font size explicitly"
-	gnome_canvas_text_set_zoom_size(GNOME_CANVAS_TEXT(self->text_item), font_size);
-	
-	g_free(font_string);
-}
-
-static void
-set_font_from_block(PangoFontDescription* font, CriaBlock* block) {
-	CriaBlock* template = cria_block_get_template(block);
-#warning "setFontFromBlock(): FIXME: get the template here and not within the block"
-	if(template) {
-		set_font_from_block(font, template);
-	}
-	
-	cria_format_set_font(font, cria_block_get_format(block));
+#endif
 }
 
 static void
 cbr_update_font(CriaBlockRenderer* self) {
 	PangoFontDescription* font = pango_font_description_copy(cria_application_get_default_font());
+	CriaLepton format = cbr_get_format(self);
+	gdouble  font_size;
+	gchar  * font_string;
 
-	set_font_from_block(font, self->block);
+	g_return_if_fail(CRIA_IS_BLOCK_RENDERER(self));
+	
+	cdebug("updateFont()", "setting to format string '%s'", cria_lepton_get(format));
+	cria_format_set_font(font, format);
 
-	update_font(self, font);
+#ifndef WITH_CCC
+	cbr_ensure_text_item(self);
+#endif
+	font_size   = 0.0 + pango_font_description_get_size(font) * MASTER_COORDINATES_PER_POINT / PANGO_SCALE;
+	font_string = pango_font_description_to_string(font);
+	
+	cdebugo(self, "updateFont()", "Font is now 0x%x", (uintptr_t)font);
+	cdebugo(self, "updateFont()", "Font: %s", font_string);
+#ifndef WITH_CCC
+	g_object_set(self->text_item,
+		     "font-desc", font,
+		     NULL);
+
+	cdebugo(self, "updateFont()", "showing canvas item at size %f", font_size);
+#warning "FIXME: make it unnecessary to set the font size explicitly"
+	gnome_canvas_text_set_zoom_size(GNOME_CANVAS_TEXT(self->text_item), font_size);
+#endif
 	pango_font_description_free(font);
+
+	g_free(font_string);
 }
 
 static void
 render_block_content(CriaBlockRenderer* self) {
 	GtkAnchorType	  anchor;
 	GtkJustification  justification;
-	GOColor		  color;
 	CriaBlock	* block = cria_block_renderer_get_block(self);
+	CriaAlignment     align;
+	CriaVAlignment    valign;
 	
 	g_return_if_fail(CRIA_IS_BLOCK(block));
+	align = cbr_get_alignment(self);
+	valign = cbr_get_valignment(self);
 
-	cdebugo(self, "renderBlock()", "%s", cria_block_get_name(block));
+#ifndef WITH_CCC
+	cbr_ensure_text_item(self);
+#endif
+	
+	if(!cbr_displays_text(self)) {
+		GOColor color = 0x666666FF;
+		g_object_set(self->text_item,
+			     "anchor", GTK_ANCHOR_CENTER,
+			     "justification", GTK_JUSTIFY_CENTER,
+			     "text", MSG_CLICK_HERE,
+			     "fill-color-rgba", color,
+			     NULL);
+		goto finish;
+	}
+
+	cdebug("renderBlock()", "the block is called '%s'", cria_slide_element_get_name(CRIA_SLIDE_ELEMENT(block)));
 
 	/* disconnect the default font changed signal */
 	if(self->default_font_changed_handler) {
@@ -518,8 +746,6 @@ render_block_content(CriaBlockRenderer* self) {
 		self->default_font_changed_handler = 0;
 	}
 	
-	color          = cria_block_get_color(block);
-
 	if(TRUE) {
 #warning "renderBlockContent(): FIXME: automatically connect the the default font"
 		self->default_font_changed_handler = g_signal_connect_swapped(cria_application_get_instance(),
@@ -528,26 +754,49 @@ render_block_content(CriaBlockRenderer* self) {
 										    self);
 	}
 
-	cdebugo(self, "renderBlock()", "alignment %i", cria_block_get_alignment(block));
+	cdebugo(self, "renderBlock()", "alignment %i", align);
 
-	anchor = get_anchor_from_alignments(cria_block_get_alignment(block), cria_block_get_valignment(block));
-	justification = get_justification_from_alignment(cria_block_get_alignment(block));
-
-	cbr_ensure_text_item(self);
+	anchor = get_anchor_from_alignments(align, valign);
+	justification = get_justification_from_alignment(align);
 
+#ifndef WITH_CCC
 	g_object_set(self->text_item,
 		     "anchor", anchor,
-		     "fill-color-rgba", color,
+		     "fill-color-rgba", cbr_get_color(self),
 		     "justification", justification,
 		     "text", cria_block_get_markup(block),
 		     NULL);
+#endif
 
+finish:
 	cbr_update_font(self);
 	cbr_update_position(self);
 
 	cdebugo(self, "renderBlock()", "done");
 }
 
+static gboolean
+cbr_focus_in_event(CriaItem* item, GdkEventFocus* ev) {
+	CriaBlockRenderer* self = CRIA_BLOCK_RENDERER(item);
+	gboolean retval = FALSE;
+	
+	cbr_disable_caret_timeout(self);
+	
+	if(self->border) {
+		g_object_set(self->border,
+			     "outline-color-gdk", &(GTK_WIDGET(GNOME_CANVAS_ITEM(item)->canvas)->style->base[GTK_STATE_SELECTED]),
+			     NULL);
+	}
+	
+	retval != CRIA_ITEM_CLASS(cria_block_renderer_parent_class)->focus_in_event(item, ev);
+	
+	render_block_content(self);
+	cbr_reset_caret(self);
+	
+	return retval;
+}
+
+#ifndef WITH_CCC
 static void
 render_block_border(CriaBlockRenderer* self) {
 	if(!self->border) {
@@ -563,16 +812,18 @@ render_block_border(CriaBlockRenderer* self) {
 		cbr_update_position(self);
 	}
 }
+#endif
 
 static void
 render_block(CriaBlockRenderer* self) {
 	render_block_content(self);
-
+#ifndef WITH_CCC
 	if(cria_item_is_interactive(CRIA_ITEM(self))) {
 		render_block_border(self);
 	} else {
 		hide_block_border(self);
 	}
+#endif
 }
 
 static void
@@ -588,7 +839,11 @@ cbr_format_changed(CriaBlockRenderer* self, CriaFormatDomain changed, CriaBlock*
 
 void
 cria_block_renderer_set_block(CriaBlockRenderer* self, CriaBlock* block) {
+#ifdef WITH_CCC
+	g_return_if_fail(CC_IS_NODE(self));
+#else
 	g_return_if_fail(CRIA_IS_ITEM(self));
+#endif
 	g_return_if_fail(CRIA_IS_BLOCK_RENDERER(self));
 	g_return_if_fail(CRIA_IS_BLOCK(block));
 
@@ -608,11 +863,13 @@ cria_block_renderer_set_block(CriaBlockRenderer* self, CriaBlock* block) {
 			         G_CALLBACK(cbr_format_changed), self);
 	
 	render_block(self);
+#ifndef WITH_CCC
 	if(cria_item_is_interactive(CRIA_ITEM(self)) &&
 	   GNOME_CANVAS_ITEM(self) == GNOME_CANVAS_ITEM(self)->canvas->focused_item)
 	{
 		cbr_update_cursor_position(self);
 	}
+#endif
 	g_object_notify(G_OBJECT(self), "block");
 }
 
@@ -636,6 +893,7 @@ void
 cria_block_renderer_show(CriaBlockRenderer* self) {
 	g_return_if_fail(CRIA_IS_BLOCK_RENDERER(self));
 
+#ifndef WITH_CCC
 	if(self->text_item) {
 		cdebugo(self, "rendererShow()", "showing item 0x%x", (uintptr_t)self->text_item);
 		gnome_canvas_item_show(self->text_item);
@@ -644,38 +902,46 @@ cria_block_renderer_show(CriaBlockRenderer* self) {
 	if(self->border && cria_item_is_interactive(CRIA_ITEM(self))) {
 		gnome_canvas_item_show(self->border);
 	}
+#endif
 }
 
 static void
 cbr_finalize(GObject* object) {
 	CriaBlockRenderer* self = CRIA_BLOCK_RENDERER(object);
 	
+	cdebug("finalize()", "Start");
 	g_signal_handlers_disconnect_by_func(self->block, cbr_format_changed, self);
 
+	cdebug("finalize()", "the block has %d references left (including the one we're going to remove now)", G_OBJECT(self->block)->ref_count);
 	g_object_unref(self->block);
 	self->block = NULL;
+	
+	cdebug("finalize()", "Finished");
 }
 
+#ifndef WITH_CCC
 static gboolean
 cbr_event(GnomeCanvasItem* item, GdkEvent* ev) {
 	CriaBlockRenderer* self = CRIA_BLOCK_RENDERER(item);
 	/* forward events to the text item */
 	return GNOME_CANVAS_ITEM_GET_CLASS(self->text_item)->event(self->text_item, ev);
 }
+#endif
 
 static void
 cria_block_renderer_class_init(CriaBlockRendererClass* self_class) {
 	GObjectClass	    * g_object_class;
+#ifndef WITH_CCC
 	GnomeCanvasItemClass* gci_class;
 	CriaItemClass	    * item_class;
+#endif
 
-	parent_class = g_type_class_peek_parent(self_class);
 	/* setting up property system */
 	g_object_class = G_OBJECT_CLASS(self_class);
 	g_object_class->finalize = cbr_finalize;
 	g_object_class->get_property = cria_block_renderer_get_property;
 	g_object_class->set_property = cria_block_renderer_set_property;
-
+#ifndef WITH_CCC
 	/* GnomeCanvasItemClass */
 	gci_class = GNOME_CANVAS_ITEM_CLASS(self_class);
 	gci_class->event = cbr_event;
@@ -685,7 +951,7 @@ cria_block_renderer_class_init(CriaBlockRendererClass* self_class) {
 	item_class->button_press_event = cbr_button_press_event;
 	item_class->focus_in_event = cbr_focus_in_event;
 	item_class->focus_out_event = cbr_focus_out_event;
-
+#endif
 	/* setting up the block view interface */
 	_cria_block_view_install_properties(g_object_class);
 }
